/*! For license information please see 223.a93208f6436a19e928eb.js.LICENSE.txt */
(self.webpackChunkchartfactor_jlab_ext=self.webpackChunkchartfactor_jlab_ext||[]).push([[223],{223:function(t,e,o){!function(t,e,o){"use strict";var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var o in e)Object.prototype.hasOwnProperty.call(e,o)&&(t[o]=e[o])},i(t,e)};function s(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Class extends value "+String(e)+" is not a constructor or null");function o(){this.constructor=t}i(t,e),t.prototype=null===e?Object.create(e):(o.prototype=e.prototype,new o)}var n,r,p=function(){function t(t){this._fn=t}return Object.defineProperty(t.prototype,"isDisposed",{get:function(){return!this._fn},enumerable:!0,configurable:!0}),t.prototype.dispose=function(){if(this._fn){var t=this._fn;this._fn=null,t()}},t}(),a=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._disposed=new o.Signal(e),e}return s(e,t),Object.defineProperty(e.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._disposed.emit(void 0),o.Signal.clearData(this))},e}(p);t.DisposableSet=function(){function t(){this._isDisposed=!1,this._items=new Set}return Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this._isDisposed},enumerable:!0,configurable:!0}),t.prototype.dispose=function(){this._isDisposed||(this._isDisposed=!0,this._items.forEach((function(t){t.dispose()})),this._items.clear())},t.prototype.contains=function(t){return this._items.has(t)},t.prototype.add=function(t){this._items.add(t)},t.prototype.remove=function(t){this._items.delete(t)},t.prototype.clear=function(){this._items.clear()},t}(),(r=t.DisposableSet||(t.DisposableSet={})).from=function(t){var o=new r;return e.each(t,(function(t){o.add(t)})),o},t.ObservableDisposableSet=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._disposed=new o.Signal(e),e}return s(e,t),Object.defineProperty(e.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._disposed.emit(void 0),o.Signal.clearData(this))},e}(t.DisposableSet),(n=t.ObservableDisposableSet||(t.ObservableDisposableSet={})).from=function(t){var o=new n;return e.each(t,(function(t){o.add(t)})),o},t.DisposableDelegate=p,t.ObservableDisposableDelegate=a,Object.defineProperty(t,"__esModule",{value:!0})}(e,o(918),o(840))}}]);